@@ -24,26 +24,21 @@
 #include "ui/base/ime/composition_text.h"
 #include "ui/base/ime/ime_bridge.h"
 #include "ui/base/ime/ime_engine_handler_interface.h"
+#include "ui/base/ime/input_method_delegate.h"
 #include "ui/base/ime/text_input_client.h"
 #include "ui/events/event.h"
 #include "ui/gfx/geometry/rect.h"
 
-namespace {
-ui::IMEEngineHandlerInterface* GetEngine() {
-  return ui::IMEBridge::Get()->GetCurrentEngineHandler();
-}
-}  // namespace
-
 namespace ui {
 
 // InputMethodChromeOS implementation -----------------------------------------
 InputMethodChromeOS::InputMethodChromeOS(
     internal::InputMethodDelegate* delegate)
-    : composing_text_(false),
+    : InputMethodBase(delegate),
+      composing_text_(false),
       composition_changed_(false),
       handling_key_event_(false),
       weak_ptr_factory_(this) {
-  SetDelegate(delegate);
   ui::IMEBridge::Get()->SetInputContextHandler(this);
 
   UpdateContextFocusState();
@@ -54,8 +49,10 @@ InputMethodChromeOS::~InputMethodChromeOS() {
   // We are dead, so we need to ask the client to stop relying on us.
   OnInputMethodChanged();
 
-  if (ui::IMEBridge::Get())
-    ui::IMEBridge::Get()->SetInputContextHandler(NULL);
+  if (ui::IMEBridge::Get() &&
+      ui::IMEBridge::Get()->GetInputContextHandler() == this) {
+    ui::IMEBridge::Get()->SetInputContextHandler(nullptr);
+  }
 }
 
 ui::EventDispatchDetails InputMethodChromeOS::DispatchKeyEvent(
@@ -64,9 +61,55 @@ ui::EventDispatchDetails InputMethodChromeOS::DispatchKeyEvent(
   DCHECK(event->IsKeyEvent());
   DCHECK(!(event->flags() & ui::EF_IS_SYNTHESIZED));
 
-  // The Caps Lock toggling has been removed from here, because now it is
-  // handled in accelerator controller.
-  // (see https://bugs.chromium.org/p/chromium/issues/detail?id=700705).
+  // For OS_CHROMEOS build of Chrome running on Linux, the IME keyboard cannot
+  // track the Caps Lock state by itself, so need to call SetCapsLockEnabled()
+  // method to reflect the Caps Lock state by the key event.
+  chromeos::input_method::InputMethodManager* manager =
+      chromeos::input_method::InputMethodManager::Get();
+  if (manager) {
+    chromeos::input_method::ImeKeyboard* keyboard = manager->GetImeKeyboard();
+    if (keyboard && event->type() == ET_KEY_PRESSED &&
+        event->key_code() != ui::VKEY_CAPITAL &&
+        keyboard->CapsLockIsEnabled() != event->IsCapsLockOn()) {
+      // Synchronize the keyboard state with event's state if they do not
+      // match. Do not synchronize for Caps Lock key because it is already
+      // handled in event rewriter.
+      keyboard->SetCapsLockEnabled(event->IsCapsLockOn());
+    }
+
+    // For JP106 language input keys, makes sure they can be passed to the app
+    // so that the VDI web apps can be supported. See https://crbug.com/816341.
+    // VKEY_CONVERT: Henkan key
+    // VKEY_NONCONVERT: Muhenkan key
+    // VKEY_DBE_SBCSCHAR/VKEY_DBE_DBCSCHAR: ZenkakuHankaku key
+    chromeos::input_method::InputMethodManager::State* state =
+        manager->GetActiveIMEState().get();
+    if (event->type() == ET_KEY_PRESSED && state) {
+      bool language_input_key = true;
+      switch (event->key_code()) {
+        case ui::VKEY_CONVERT:
+          state->ChangeInputMethodToJpIme();
+          break;
+        case ui::VKEY_NONCONVERT:
+          state->ChangeInputMethodToJpKeyboard();
+          break;
+        case ui::VKEY_DBE_SBCSCHAR:
+        case ui::VKEY_DBE_DBCSCHAR:
+          state->ToggleInputMethodForJpIme();
+          break;
+        default:
+          language_input_key = false;
+          break;
+      }
+      if (language_input_key) {
+        // Dispatches the event to app/blink.
+        // TODO(shuchen): Eventually, the language input keys should be handed
+        // over to the IME extension to process. And IMF can handle if the IME
+        // extension didn't handle.
+        return DispatchKeyEventPostIME(event, std::move(ack_callback));
+      }
+    }
+  }
 
   // If |context_| is not usable, then we can only dispatch the key event as is.
   // We only dispatch the key event to input method when the |context_| is an
@@ -74,45 +117,32 @@ ui::EventDispatchDetails InputMethodChromeOS::DispatchKeyEvent(
   // Note: We need to send the key event to ibus even if the |context_| is not
   // enabled, so that ibus can have a chance to enable the |context_|.
   if (!IsNonPasswordInputFieldFocused() || !GetEngine()) {
-    ui::EventDispatchDetails dispatch_details;
     if (event->type() == ET_KEY_PRESSED) {
       if (ExecuteCharacterComposer(*event)) {
         // Treating as PostIME event if character composer handles key event and
         // generates some IME event,
-        dispatch_details = ProcessKeyEventPostIME(event, true);
-        if (ack_callback)
-          std::move(ack_callback).Run(true);
-        return dispatch_details;
+        return ProcessKeyEventPostIME(event, std::move(ack_callback), false,
+                                      true);
       }
-      dispatch_details = ProcessUnfilteredKeyPressEvent(event);
-    } else {
-      dispatch_details = DispatchKeyEventPostIME(event);
+      return ProcessUnfilteredKeyPressEvent(event, std::move(ack_callback));
     }
-    if (ack_callback)
-      std::move(ack_callback).Run(false);
-    return dispatch_details;
+    return DispatchKeyEventPostIME(event, std::move(ack_callback));
   }
 
   handling_key_event_ = true;
   if (GetEngine()->IsInterestedInKeyEvent()) {
-    ui::IMEEngineHandlerInterface::KeyEventDoneCallback callback = base::Bind(
-        &InputMethodChromeOS::KeyEventDoneCallback,
-        weak_ptr_factory_.GetWeakPtr(),
-        // Pass the ownership of the new copied event.
-        base::Owned(new ui::KeyEvent(*event)), Passed(&ack_callback));
-    GetEngine()->ProcessKeyEvent(*event, callback);
+    ui::IMEEngineHandlerInterface::KeyEventDoneCallback callback =
+        base::BindOnce(&InputMethodChromeOS::KeyEventDoneCallback,
+                       weak_ptr_factory_.GetWeakPtr(),
+                       // Pass the ownership of the new copied event.
+                       base::Owned(new ui::KeyEvent(*event)),
+                       std::move(ack_callback));
+    GetEngine()->ProcessKeyEvent(*event, std::move(callback));
     return ui::EventDispatchDetails();
   }
-
   return ProcessKeyEventDone(event, std::move(ack_callback), false);
 }
 
-bool InputMethodChromeOS::OnUntranslatedIMEMessage(
-    const base::NativeEvent& event,
-    NativeEventResult* result) {
-  return false;
-}
-
 void InputMethodChromeOS::KeyEventDoneCallback(ui::KeyEvent* event,
                                                AckCallback ack_callback,
                                                bool is_handled) {
@@ -136,14 +166,11 @@ ui::EventDispatchDetails InputMethodChromeOS::ProcessKeyEventDone(
       is_handled = ExecuteCharacterComposer(*event);
     }
   }
-
-  if (ack_callback)
-    std::move(ack_callback).Run(is_handled);
-
   ui::EventDispatchDetails details;
-  if (event->type() == ET_KEY_PRESSED || event->type() == ET_KEY_RELEASED)
-    details = ProcessKeyEventPostIME(event, is_handled);
-
+  if (event->type() == ET_KEY_PRESSED || event->type() == ET_KEY_RELEASED) {
+    details = ProcessKeyEventPostIME(event, std::move(ack_callback), false,
+                                     is_handled);
+  }
   handling_key_event_ = false;
   return details;
 }
@@ -167,7 +194,8 @@ void InputMethodChromeOS::OnTextInputTypeChanged(
     // The focus in to or out from password field should also notify engine.
     engine->FocusOut();
     ui::IMEEngineHandlerInterface::InputContext context(
-        GetTextInputType(), GetTextInputMode(), GetTextInputFlags());
+        GetTextInputType(), GetTextInputMode(), GetTextInputFlags(),
+        GetClientFocusReason(), GetClientShouldDoLearning());
     engine->FocusIn(context);
   }
 
@@ -253,6 +281,17 @@ bool InputMethodChromeOS::IsCandidatePopupOpen() const {
   return false;
 }
 
+InputMethodKeyboardController*
+InputMethodChromeOS::GetInputMethodKeyboardController() {
+  chromeos::input_method::InputMethodManager* manager =
+      chromeos::input_method::InputMethodManager::Get();
+  if (manager) {
+    if (auto* controller = manager->GetInputMethodKeyboardController())
+      return controller;
+  }
+  return InputMethodBase::GetInputMethodKeyboardController();
+}
+
 void InputMethodChromeOS::OnWillChangeFocusedClient(
     TextInputClient* focused_before,
     TextInputClient* focused) {
@@ -272,7 +311,8 @@ void InputMethodChromeOS::OnDidChangeFocusedClient(
 
   if (GetEngine()) {
     ui::IMEEngineHandlerInterface::InputContext context(
-        GetTextInputType(), GetTextInputMode(), GetTextInputFlags());
+        GetTextInputType(), GetTextInputMode(), GetTextInputFlags(),
+        GetClientFocusReason(), GetClientShouldDoLearning());
     GetEngine()->FocusIn(context);
   }
 }
@@ -289,7 +329,7 @@ void InputMethodChromeOS::ResetContext() {
   if (!IsNonPasswordInputFieldFocused() || !GetTextInputClient())
     return;
 
-  composition_.Clear();
+  composition_ = CompositionText();
   result_text_.clear();
   composing_text_ = false;
   composition_changed_ = false;
@@ -316,7 +356,8 @@ void InputMethodChromeOS::UpdateContextFocusState() {
     candidate_window->FocusStateChanged(IsNonPasswordInputFieldFocused());
 
   ui::IMEEngineHandlerInterface::InputContext context(
-      GetTextInputType(), GetTextInputMode(), GetTextInputFlags());
+      GetTextInputType(), GetTextInputMode(), GetTextInputFlags(),
+      GetClientFocusReason(), GetClientShouldDoLearning());
   ui::IMEBridge::Get()->SetCurrentInputContext(context);
 
   if (!IsTextInputTypeNone())
@@ -325,55 +366,62 @@ void InputMethodChromeOS::UpdateContextFocusState() {
 
 ui::EventDispatchDetails InputMethodChromeOS::ProcessKeyEventPostIME(
     ui::KeyEvent* event,
+    AckCallback ack_callback,
+    bool skip_process_filtered,
     bool handled) {
   TextInputClient* client = GetTextInputClient();
   if (!client) {
     // As ibus works asynchronously, there is a chance that the focused client
     // loses focus before this method gets called.
-    return DispatchKeyEventPostIME(event);
+    return DispatchKeyEventPostIME(event, std::move(ack_callback));
   }
 
   ui::EventDispatchDetails dispatch_details;
-  if (event->type() == ET_KEY_PRESSED && handled) {
-    dispatch_details = ProcessFilteredKeyPressEvent(event);
-    if (event->stopped_propagation()) {
-      ResetContext();
-      return dispatch_details;
-    }
-    if (dispatch_details.dispatcher_destroyed ||
-        dispatch_details.target_destroyed)
-      return dispatch_details;
-  }
+  if (event->type() == ET_KEY_PRESSED && handled && !skip_process_filtered)
+    return ProcessFilteredKeyPressEvent(event, std::move(ack_callback));
 
   // In case the focus was changed by the key event. The |context_| should have
   // been reset when the focused window changed.
-  if (client != GetTextInputClient())
+  if (client != GetTextInputClient()) {
+    if (ack_callback)
+      std::move(ack_callback).Run(false);
     return dispatch_details;
-
+  }
   if (HasInputMethodResult())
     ProcessInputMethodResult(event, handled);
 
   // In case the focus was changed when sending input method results to the
   // focused window.
-  if (client != GetTextInputClient())
+  if (client != GetTextInputClient()) {
+    if (ack_callback)
+      std::move(ack_callback).Run(false);
     return dispatch_details;
+  }
 
-  if (handled)
+  if (handled) {
+    if (ack_callback)
+      std::move(ack_callback).Run(true);
     return dispatch_details;  // IME handled the key event. do not forward.
+  }
 
   if (event->type() == ET_KEY_PRESSED)
-    return ProcessUnfilteredKeyPressEvent(event);
+    return ProcessUnfilteredKeyPressEvent(event, std::move(ack_callback));
 
   if (event->type() == ET_KEY_RELEASED)
-    return DispatchKeyEventPostIME(event);
-
+    return DispatchKeyEventPostIME(event, std::move(ack_callback));
   return dispatch_details;
 }
 
 ui::EventDispatchDetails InputMethodChromeOS::ProcessFilteredKeyPressEvent(
-    ui::KeyEvent* event) {
+    ui::KeyEvent* event,
+    AckCallback ack_callback) {
+  auto callback = base::Bind(
+      &InputMethodChromeOS::PostProcessFilteredKeyPressEvent,
+      weak_ptr_factory_.GetWeakPtr(), base::Owned(new ui::KeyEvent(*event)),
+      GetTextInputClient(), Passed(&ack_callback));
+
   if (NeedInsertChar())
-    return DispatchKeyEventPostIME(event);
+    return DispatchKeyEventPostIME(event, std::move(callback));
 
   ui::KeyEvent fabricated_event(ET_KEY_PRESSED,
                                 VKEY_PROCESSKEY,
@@ -382,19 +430,52 @@ ui::EventDispatchDetails InputMethodChromeOS::ProcessFilteredKeyPressEvent(
                                 event->GetDomKey(),
                                 event->time_stamp());
   ui::EventDispatchDetails dispatch_details =
-      DispatchKeyEventPostIME(&fabricated_event);
+      DispatchKeyEventPostIME(&fabricated_event, std::move(callback));
   if (fabricated_event.stopped_propagation())
     event->StopPropagation();
   return dispatch_details;
 }
 
-ui::EventDispatchDetails InputMethodChromeOS::ProcessUnfilteredKeyPressEvent(
-    ui::KeyEvent* event) {
-  const TextInputClient* prev_client = GetTextInputClient();
-  ui::EventDispatchDetails dispatch_details = DispatchKeyEventPostIME(event);
-  if (event->stopped_propagation()) {
+void InputMethodChromeOS::PostProcessFilteredKeyPressEvent(
+    ui::KeyEvent* event,
+    TextInputClient* prev_client,
+    AckCallback ack_callback,
+    bool stopped_propagation) {
+  // In case the focus was changed by the key event.
+  if (GetTextInputClient() != prev_client)
+    return;
+
+  if (stopped_propagation) {
     ResetContext();
-    return dispatch_details;
+    if (ack_callback)
+      std::move(ack_callback).Run(true);
+    return;
+  }
+  ignore_result(
+      ProcessKeyEventPostIME(event, std::move(ack_callback), true, true));
+}
+
+ui::EventDispatchDetails InputMethodChromeOS::ProcessUnfilteredKeyPressEvent(
+    ui::KeyEvent* event,
+    AckCallback ack_callback) {
+  return DispatchKeyEventPostIME(
+      event,
+      base::BindOnce(&InputMethodChromeOS::PostProcessUnfilteredKeyPressEvent,
+                     weak_ptr_factory_.GetWeakPtr(),
+                     base::Owned(new ui::KeyEvent(*event)),
+                     GetTextInputClient(), std::move(ack_callback)));
+}
+
+void InputMethodChromeOS::PostProcessUnfilteredKeyPressEvent(
+    ui::KeyEvent* event,
+    TextInputClient* prev_client,
+    AckCallback ack_callback,
+    bool stopped_propagation) {
+  if (stopped_propagation) {
+    ResetContext();
+    if (ack_callback)
+      std::move(ack_callback).Run(false);
+    return;
   }
 
   // We shouldn't dispatch the character anymore if the key event dispatch
@@ -405,8 +486,11 @@ ui::EventDispatchDetails InputMethodChromeOS::ProcessUnfilteredKeyPressEvent(
   //    page.
   // We should return here not to send the Tab key event to RWHV.
   TextInputClient* client = GetTextInputClient();
-  if (!client || client != prev_client)
-    return dispatch_details;
+  if (!client || client != prev_client) {
+    if (ack_callback)
+      std::move(ack_callback).Run(false);
+    return;
+  }
 
   // If a key event was not filtered by |context_| and |character_composer_|,
   // then it means the key event didn't generate any result text. So we need
@@ -414,7 +498,9 @@ ui::EventDispatchDetails InputMethodChromeOS::ProcessUnfilteredKeyPressEvent(
   uint16_t ch = event->GetCharacter();
   if (ch)
     client->InsertChar(*event);
-  return dispatch_details;
+
+  if (ack_callback)
+    std::move(ack_callback).Run(false);
 }
 
 void InputMethodChromeOS::ProcessInputMethodResult(ui::KeyEvent* event,
@@ -426,7 +512,7 @@ void InputMethodChromeOS::ProcessInputMethodResult(ui::KeyEvent* event,
     if (handled && NeedInsertChar()) {
       for (base::string16::const_iterator i = result_text_.begin();
            i != result_text_.end(); ++i) {
-        ui::KeyEvent ch_event(*event);
+        KeyEvent ch_event(ET_KEY_PRESSED, VKEY_UNKNOWN, EF_NONE);
         ch_event.set_character(*i);
         client->InsertChar(ch_event);
       }
@@ -534,7 +620,7 @@ void InputMethodChromeOS::UpdateCompositionText(const CompositionText& text,
       GetTextInputClient()->SetCompositionText(composition_);
     SendFakeProcessKeyEvent(false);
     composition_changed_ = false;
-    composition_.Clear();
+    composition_ = CompositionText();
   }
 }
 
@@ -544,7 +630,7 @@ void InputMethodChromeOS::HidePreeditText() {
 
   // Intentionally leaves |composing_text_| unchanged.
   composition_changed_ = true;
-  composition_.Clear();
+  composition_ = CompositionText();
 
   if (!handling_key_event_) {
     TextInputClient* client = GetTextInputClient();
@@ -588,7 +674,7 @@ void InputMethodChromeOS::ExtractCompositionText(
     const CompositionText& text,
     uint32_t cursor_position,
     CompositionText* out_composition) const {
-  out_composition->Clear();
+  *out_composition = CompositionText();
   out_composition->text = text.text;
 
   if (out_composition->text.empty())
@@ -613,17 +699,19 @@ void InputMethodChromeOS::ExtractCompositionText(
 
   out_composition->selection = gfx::Range(cursor_offset);
 
-  const CompositionUnderlines text_underlines = text.underlines;
-  if (!text_underlines.empty()) {
-    for (size_t i = 0; i < text_underlines.size(); ++i) {
-      const uint32_t start = text_underlines[i].start_offset;
-      const uint32_t end = text_underlines[i].end_offset;
+  const ImeTextSpans text_ime_text_spans = text.ime_text_spans;
+  if (!text_ime_text_spans.empty()) {
+    for (size_t i = 0; i < text_ime_text_spans.size(); ++i) {
+      const uint32_t start = text_ime_text_spans[i].start_offset;
+      const uint32_t end = text_ime_text_spans[i].end_offset;
       if (start >= end)
         continue;
-      CompositionUnderline underline(
-          char16_offsets[start], char16_offsets[end], text_underlines[i].color,
-          text_underlines[i].thick, text_underlines[i].background_color);
-      out_composition->underlines.push_back(underline);
+      ImeTextSpan ime_text_span(ui::ImeTextSpan::Type::kComposition,
+                                char16_offsets[start], char16_offsets[end],
+                                text_ime_text_spans[i].thickness,
+                                text_ime_text_spans[i].background_color);
+      ime_text_span.underline_color = text_ime_text_spans[i].underline_color;
+      out_composition->ime_text_spans.push_back(ime_text_span);
     }
   }
 
@@ -631,29 +719,29 @@ void InputMethodChromeOS::ExtractCompositionText(
   if (text.selection.start() < text.selection.end()) {
     const uint32_t start = text.selection.start();
     const uint32_t end = text.selection.end();
-    CompositionUnderline underline(char16_offsets[start],
-                                   char16_offsets[end],
-                                   SK_ColorBLACK,
-                                   true /* thick */,
-                                   SK_ColorTRANSPARENT);
-    out_composition->underlines.push_back(underline);
+    ImeTextSpan ime_text_span(ui::ImeTextSpan::Type::kComposition,
+                              char16_offsets[start], char16_offsets[end],
+                              ui::ImeTextSpan::Thickness::kThick,
+                              SK_ColorTRANSPARENT);
+    out_composition->ime_text_spans.push_back(ime_text_span);
 
-    // If the cursor is at start or end of this underline, then we treat
+    // If the cursor is at start or end of this ime_text_span, then we treat
     // it as the selection range as well, but make sure to set the cursor
     // position to the selection end.
-    if (underline.start_offset == cursor_offset) {
-      out_composition->selection.set_start(underline.end_offset);
+    if (ime_text_span.start_offset == cursor_offset) {
+      out_composition->selection.set_start(ime_text_span.end_offset);
       out_composition->selection.set_end(cursor_offset);
-    } else if (underline.end_offset == cursor_offset) {
-      out_composition->selection.set_start(underline.start_offset);
+    } else if (ime_text_span.end_offset == cursor_offset) {
+      out_composition->selection.set_start(ime_text_span.start_offset);
       out_composition->selection.set_end(cursor_offset);
     }
   }
 
-  // Use a black thin underline by default.
-  if (out_composition->underlines.empty()) {
-    out_composition->underlines.push_back(CompositionUnderline(
-        0, length, SK_ColorBLACK, false /* thick */, SK_ColorTRANSPARENT));
+  // Use a thin underline with text color by default.
+  if (out_composition->ime_text_spans.empty()) {
+    out_composition->ime_text_spans.push_back(
+        ImeTextSpan(ui::ImeTextSpan::Type::kComposition, 0, length,
+                    ui::ImeTextSpan::Thickness::kThin, SK_ColorTRANSPARENT));
   }
 }
 
@@ -666,4 +754,9 @@ bool InputMethodChromeOS::IsInputFieldFocused() {
   return GetTextInputType() != TEXT_INPUT_TYPE_NONE;
 }
 
+TextInputClient::FocusReason InputMethodChromeOS::GetClientFocusReason() const {
+  TextInputClient* client = GetTextInputClient();
+  return client ? client->GetFocusReason() : TextInputClient::FOCUS_REASON_NONE;
+}
+
 }  // namespace ui
