@@ -25,10 +25,6 @@
 namespace ui {
 namespace {
 
-// Extra number of chars before and after selection (or composition) range which
-// is returned to IME for improving conversion accuracy.
-static const size_t kExtraNumberOfChars = 20;
-
 ui::EventDispatchDetails DispatcherDestroyedDetails() {
   ui::EventDispatchDetails dispatcher_details;
   dispatcher_details.dispatcher_destroyed = true;
@@ -39,14 +35,12 @@ ui::EventDispatchDetails DispatcherDestroyedDetails() {
 
 InputMethodWin::InputMethodWin(internal::InputMethodDelegate* delegate,
                                HWND toplevel_window_handle)
-    : toplevel_window_handle_(toplevel_window_handle),
+    : InputMethodWinBase(delegate, toplevel_window_handle),
       pending_requested_direction_(base::i18n::UNKNOWN_DIRECTION),
-      accept_carriage_return_(false),
       enabled_(false),
       is_candidate_popup_open_(false),
       composing_window_handle_(NULL),
       weak_ptr_factory_(this) {
-  SetDelegate(delegate);
   imm32_manager_.SetInputLanguage();
 }
 
@@ -58,7 +52,7 @@ void InputMethodWin::OnFocus() {
 }
 
 bool InputMethodWin::OnUntranslatedIMEMessage(
-    const base::NativeEvent& event,
+    const MSG event,
     InputMethod::NativeEventResult* result) {
   LRESULT original_result = 0;
   BOOL handled = FALSE;
@@ -103,13 +97,10 @@ bool InputMethodWin::OnUntranslatedIMEMessage(
 }
 
 ui::EventDispatchDetails InputMethodWin::DispatchKeyEvent(ui::KeyEvent* event) {
-  if (!event->HasNativeEvent())
-    return DispatchFabricatedKeyEvent(event);
-
-  const base::NativeEvent& native_key_event = event->native_event();
-  BOOL handled = FALSE;
+  MSG native_key_event = MSGFromKeyEvent(event);
   if (native_key_event.message == WM_CHAR) {
     auto ref = weak_ptr_factory_.GetWeakPtr();
+    BOOL handled = FALSE;
     OnChar(native_key_event.hwnd, native_key_event.message,
            native_key_event.wParam, native_key_event.lParam, native_key_event,
            &handled);
@@ -146,10 +137,8 @@ ui::EventDispatchDetails InputMethodWin::DispatchKeyEvent(ui::KeyEvent* event) {
   // Handles ctrl-shift key to change text direction and layout alignment.
   if (ui::IMM32Manager::IsRTLKeyboardLayoutInstalled() &&
       !IsTextInputTypeNone()) {
-    // TODO: shouldn't need to generate a KeyEvent here.
-    const ui::KeyEvent key(native_key_event);
-    ui::KeyboardCode code = key.key_code();
-    if (key.type() == ui::ET_KEY_PRESSED) {
+    ui::KeyboardCode code = event->key_code();
+    if (event->type() == ui::ET_KEY_PRESSED) {
       if (code == ui::VKEY_SHIFT) {
         base::i18n::TextDirection dir;
         if (ui::IMM32Manager::IsCtrlShiftPressed(&dir))
@@ -157,7 +146,7 @@ ui::EventDispatchDetails InputMethodWin::DispatchKeyEvent(ui::KeyEvent* event) {
       } else if (code != ui::VKEY_CONTROL) {
         pending_requested_direction_ = base::i18n::UNKNOWN_DIRECTION;
       }
-    } else if (key.type() == ui::ET_KEY_RELEASED &&
+    } else if (event->type() == ui::ET_KEY_RELEASED &&
                (code == ui::VKEY_SHIFT || code == ui::VKEY_CONTROL) &&
                pending_requested_direction_ != base::i18n::UNKNOWN_DIRECTION) {
       GetTextInputClient()->ChangeTextDirectionAndLayoutAlignment(
@@ -178,11 +167,12 @@ ui::EventDispatchDetails InputMethodWin::DispatchKeyEvent(ui::KeyEvent* event) {
   // messages have been combined in the event processing flow.
   if (char_msgs.size() <= 1 && GetEngine() &&
       GetEngine()->IsInterestedInKeyEvent()) {
-    ui::IMEEngineHandlerInterface::KeyEventDoneCallback callback = base::Bind(
-        &InputMethodWin::ProcessKeyEventDone, weak_ptr_factory_.GetWeakPtr(),
-        base::Owned(new ui::KeyEvent(*event)),
-        base::Owned(new std::vector<MSG>(char_msgs)));
-    GetEngine()->ProcessKeyEvent(*event, callback);
+    ui::IMEEngineHandlerInterface::KeyEventDoneCallback callback =
+        base::BindOnce(&InputMethodWin::ProcessKeyEventDone,
+                       weak_ptr_factory_.GetWeakPtr(),
+                       base::Owned(new ui::KeyEvent(*event)),
+                       base::Owned(new std::vector<MSG>(char_msgs)));
+    GetEngine()->ProcessKeyEvent(*event, std::move(callback));
     return ui::EventDispatchDetails();
   }
 
@@ -313,44 +303,7 @@ void InputMethodWin::OnDidChangeFocusedClient(
     // bounds has not changed.
     OnCaretBoundsChanged(focused);
   }
-  if (focused_before != focused)
-    accept_carriage_return_ = false;
-}
-
-LRESULT InputMethodWin::OnChar(HWND window_handle,
-                               UINT message,
-                               WPARAM wparam,
-                               LPARAM lparam,
-                               const base::NativeEvent& event,
-                               BOOL* handled) {
-  *handled = TRUE;
-
-  // We need to send character events to the focused text input client event if
-  // its text input type is ui::TEXT_INPUT_TYPE_NONE.
-  if (GetTextInputClient()) {
-    const base::char16 kCarriageReturn = L'\r';
-    const base::char16 ch = static_cast<base::char16>(wparam);
-    // A mask to determine the previous key state from |lparam|. The value is 1
-    // if the key is down before the message is sent, or it is 0 if the key is
-    // up.
-    const uint32_t kPrevKeyDownBit = 0x40000000;
-    if (ch == kCarriageReturn && !(lparam & kPrevKeyDownBit))
-      accept_carriage_return_ = true;
-    // Conditionally ignore '\r' events to work around crbug.com/319100.
-    // TODO(yukawa, IME): Figure out long-term solution.
-    if (ch != kCarriageReturn || accept_carriage_return_) {
-      ui::KeyEvent char_event(event);
-      GetTextInputClient()->InsertChar(char_event);
-    }
-  }
-
-  // Explicitly show the system menu at a good location on [Alt]+[Space].
-  // Note: Setting |handled| to FALSE for DefWindowProc triggering of the system
-  //       menu causes undesirable titlebar artifacts in the classic theme.
-  if (message == WM_SYSCHAR && wparam == VK_SPACE)
-    gfx::ShowSystemMenu(window_handle);
-
-  return 0;
+  InputMethodWinBase::OnDidChangeFocusedClient(focused_before, focused);
 }
 
 LRESULT InputMethodWin::OnImeSetContext(HWND window_handle,
@@ -481,188 +434,6 @@ LRESULT InputMethodWin::OnImeNotify(UINT message,
   return 0;
 }
 
-LRESULT InputMethodWin::OnImeRequest(UINT message,
-                                     WPARAM wparam,
-                                     LPARAM lparam,
-                                     BOOL* handled) {
-  *handled = FALSE;
-
-  // Should not receive WM_IME_REQUEST message, if IME is disabled.
-  const ui::TextInputType type = GetTextInputType();
-  if (type == ui::TEXT_INPUT_TYPE_NONE ||
-      type == ui::TEXT_INPUT_TYPE_PASSWORD) {
-    return 0;
-  }
-
-  switch (wparam) {
-    case IMR_RECONVERTSTRING:
-      *handled = TRUE;
-      return OnReconvertString(reinterpret_cast<RECONVERTSTRING*>(lparam));
-    case IMR_DOCUMENTFEED:
-      *handled = TRUE;
-      return OnDocumentFeed(reinterpret_cast<RECONVERTSTRING*>(lparam));
-    case IMR_QUERYCHARPOSITION:
-      *handled = TRUE;
-      return OnQueryCharPosition(reinterpret_cast<IMECHARPOSITION*>(lparam));
-    default:
-      return 0;
-  }
-}
-
-LRESULT InputMethodWin::OnDocumentFeed(RECONVERTSTRING* reconv) {
-  ui::TextInputClient* client = GetTextInputClient();
-  if (!client)
-    return 0;
-
-  gfx::Range text_range;
-  if (!client->GetTextRange(&text_range) || text_range.is_empty())
-    return 0;
-
-  bool result = false;
-  gfx::Range target_range;
-  if (client->HasCompositionText())
-    result = client->GetCompositionTextRange(&target_range);
-
-  if (!result || target_range.is_empty()) {
-    if (!client->GetSelectionRange(&target_range) ||
-        !target_range.IsValid()) {
-      return 0;
-    }
-  }
-
-  if (!text_range.Contains(target_range))
-    return 0;
-
-  if (target_range.GetMin() - text_range.start() > kExtraNumberOfChars)
-    text_range.set_start(target_range.GetMin() - kExtraNumberOfChars);
-
-  if (text_range.end() - target_range.GetMax() > kExtraNumberOfChars)
-    text_range.set_end(target_range.GetMax() + kExtraNumberOfChars);
-
-  size_t len = text_range.length();
-  size_t need_size = sizeof(RECONVERTSTRING) + len * sizeof(WCHAR);
-
-  if (!reconv)
-    return need_size;
-
-  if (reconv->dwSize < need_size)
-    return 0;
-
-  base::string16 text;
-  if (!GetTextInputClient()->GetTextFromRange(text_range, &text))
-    return 0;
-  DCHECK_EQ(text_range.length(), text.length());
-
-  reconv->dwVersion = 0;
-  reconv->dwStrLen = len;
-  reconv->dwStrOffset = sizeof(RECONVERTSTRING);
-  reconv->dwCompStrLen =
-      client->HasCompositionText() ? target_range.length() : 0;
-  reconv->dwCompStrOffset =
-      (target_range.GetMin() - text_range.start()) * sizeof(WCHAR);
-  reconv->dwTargetStrLen = target_range.length();
-  reconv->dwTargetStrOffset = reconv->dwCompStrOffset;
-
-  memcpy((char*)reconv + sizeof(RECONVERTSTRING),
-         text.c_str(), len * sizeof(WCHAR));
-
-  // According to Microsoft API document, IMR_RECONVERTSTRING and
-  // IMR_DOCUMENTFEED should return reconv, but some applications return
-  // need_size.
-  return reinterpret_cast<LRESULT>(reconv);
-}
-
-LRESULT InputMethodWin::OnReconvertString(RECONVERTSTRING* reconv) {
-  ui::TextInputClient* client = GetTextInputClient();
-  if (!client)
-    return 0;
-
-  // If there is a composition string already, we don't allow reconversion.
-  if (client->HasCompositionText())
-    return 0;
-
-  gfx::Range text_range;
-  if (!client->GetTextRange(&text_range) || text_range.is_empty())
-    return 0;
-
-  gfx::Range selection_range;
-  if (!client->GetSelectionRange(&selection_range) ||
-      selection_range.is_empty()) {
-    return 0;
-  }
-
-  DCHECK(text_range.Contains(selection_range));
-
-  size_t len = selection_range.length();
-  size_t need_size = sizeof(RECONVERTSTRING) + len * sizeof(WCHAR);
-
-  if (!reconv)
-    return need_size;
-
-  if (reconv->dwSize < need_size)
-    return 0;
-
-  // TODO(penghuang): Return some extra context to help improve IME's
-  // reconversion accuracy.
-  base::string16 text;
-  if (!GetTextInputClient()->GetTextFromRange(selection_range, &text))
-    return 0;
-  DCHECK_EQ(selection_range.length(), text.length());
-
-  reconv->dwVersion = 0;
-  reconv->dwStrLen = len;
-  reconv->dwStrOffset = sizeof(RECONVERTSTRING);
-  reconv->dwCompStrLen = len;
-  reconv->dwCompStrOffset = 0;
-  reconv->dwTargetStrLen = len;
-  reconv->dwTargetStrOffset = 0;
-
-  memcpy(reinterpret_cast<char*>(reconv) + sizeof(RECONVERTSTRING),
-         text.c_str(), len * sizeof(WCHAR));
-
-  // According to Microsoft API document, IMR_RECONVERTSTRING and
-  // IMR_DOCUMENTFEED should return reconv, but some applications return
-  // need_size.
-  return reinterpret_cast<LRESULT>(reconv);
-}
-
-LRESULT InputMethodWin::OnQueryCharPosition(IMECHARPOSITION* char_positon) {
-  if (!char_positon)
-    return 0;
-
-  if (char_positon->dwSize < sizeof(IMECHARPOSITION))
-    return 0;
-
-  ui::TextInputClient* client = GetTextInputClient();
-  if (!client)
-    return 0;
-
-  // Tentatively assume that the returned value from |client| is DIP (Density
-  // Independent Pixel). See the comment in text_input_client.h and
-  // http://crbug.com/360334.
-  gfx::Rect dip_rect;
-  if (client->HasCompositionText()) {
-    if (!client->GetCompositionCharacterBounds(char_positon->dwCharPos,
-                                               &dip_rect)) {
-      return 0;
-    }
-  } else {
-    // If there is no composition and the first character is queried, returns
-    // the caret bounds. This behavior is the same to that of RichEdit control.
-    if (char_positon->dwCharPos != 0)
-      return 0;
-    dip_rect = client->GetCaretBounds();
-  }
-  const gfx::Rect rect =
-      display::win::ScreenWin::DIPToScreenRect(toplevel_window_handle_,
-                                               dip_rect);
-
-  char_positon->pt.x = rect.x();
-  char_positon->pt.y = rect.y();
-  char_positon->cLineHeight = rect.height();
-  return 1;  // returns non-zero value when succeeded.
-}
-
 void InputMethodWin::RefreshInputLanguage() {
   TextInputType type_original = GetTextInputType();
   imm32_manager_.SetInputLanguage();
@@ -677,33 +448,6 @@ void InputMethodWin::RefreshInputLanguage() {
   }
 }
 
-bool InputMethodWin::IsWindowFocused(const TextInputClient* client) const {
-  if (!client)
-    return false;
-  // When Aura is enabled, |attached_window_handle| should always be a top-level
-  // window. So we can safely assume that |attached_window_handle| is ready for
-  // receiving keyboard input as long as it is an active window. This works well
-  // even when the |attached_window_handle| becomes active but has not received
-  // WM_FOCUS yet.
-  return toplevel_window_handle_ &&
-      GetActiveWindow() == toplevel_window_handle_;
-}
-
-ui::EventDispatchDetails InputMethodWin::DispatchFabricatedKeyEvent(
-    ui::KeyEvent* event) {
-  // The key event if from calling input.ime.sendKeyEvent or test.
-  ui::EventDispatchDetails details = DispatchKeyEventPostIME(event);
-  if (details.dispatcher_destroyed || details.target_destroyed ||
-      event->stopped_propagation()) {
-    return details;
-  }
-
-  if ((event->is_char() || event->GetDomKey().IsCharacter()) &&
-      event->type() == ui::ET_KEY_PRESSED && GetTextInputClient())
-    GetTextInputClient()->InsertChar(*event);
-  return details;
-}
-
 void InputMethodWin::ConfirmCompositionText() {
   if (composing_window_handle_)
     imm32_manager_.CleanupComposition(composing_window_handle_);
@@ -750,7 +494,8 @@ void InputMethodWin::UpdateIMEState() {
   const TextInputType old_text_input_type =
       ui::IMEBridge::Get()->GetCurrentInputContext().type;
   ui::IMEEngineHandlerInterface::InputContext context(
-      GetTextInputType(), GetTextInputMode(), GetTextInputFlags());
+      GetTextInputType(), GetTextInputMode(), GetTextInputFlags(),
+      ui::TextInputClient::FOCUS_REASON_OTHER, GetClientShouldDoLearning());
   ui::IMEBridge::Get()->SetCurrentInputContext(context);
 
   ui::IMEEngineHandlerInterface* engine = GetEngine();
