@@ -6,7 +6,6 @@
 
 #include "base/bind.h"
 #include "base/location.h"
-#include "base/memory/ptr_util.h"
 #include "base/threading/thread_task_runner_handle.h"
 #include "ui/display/types/display_mode.h"
 #include "ui/display/types/display_snapshot.h"
@@ -34,13 +33,13 @@ void DrmDisplayHost::UpdateDisplaySnapshot(
 
 void DrmDisplayHost::Configure(const display::DisplayMode* mode,
                                const gfx::Point& origin,
-                               const display::ConfigureCallback& callback) {
+                               display::ConfigureCallback callback) {
   if (is_dummy_) {
-    callback.Run(true);
+    std::move(callback).Run(true);
     return;
   }
 
-  configure_callback_ = callback;
+  configure_callback_ = std::move(callback);
   bool status = false;
   if (mode) {
     status = sender_->GpuConfigureNativeDisplay(
@@ -56,7 +55,7 @@ void DrmDisplayHost::Configure(const display::DisplayMode* mode,
 void DrmDisplayHost::OnDisplayConfigured(bool status) {
   if (!configure_callback_.is_null()) {
     base::ThreadTaskRunnerHandle::Get()->PostTask(
-        FROM_HERE, base::Bind(configure_callback_, status));
+        FROM_HERE, base::BindOnce(std::move(configure_callback_), status));
   } else {
     LOG(ERROR) << "Got unexpected event for display "
                << snapshot_->display_id();
@@ -65,9 +64,8 @@ void DrmDisplayHost::OnDisplayConfigured(bool status) {
   configure_callback_.Reset();
 }
 
-void DrmDisplayHost::GetHDCPState(
-    const display::GetHDCPStateCallback& callback) {
-  get_hdcp_callback_ = callback;
+void DrmDisplayHost::GetHDCPState(display::GetHDCPStateCallback callback) {
+  get_hdcp_callback_ = std::move(callback);
   if (!sender_->GpuGetHDCPState(snapshot_->display_id()))
     OnHDCPStateReceived(false, display::HDCP_STATE_UNDESIRED);
 }
@@ -76,7 +74,8 @@ void DrmDisplayHost::OnHDCPStateReceived(bool status,
                                          display::HDCPState state) {
   if (!get_hdcp_callback_.is_null()) {
     base::ThreadTaskRunnerHandle::Get()->PostTask(
-        FROM_HERE, base::Bind(get_hdcp_callback_, status, state));
+        FROM_HERE,
+        base::BindOnce(std::move(get_hdcp_callback_), status, state));
   } else {
     LOG(ERROR) << "Got unexpected event for display "
                << snapshot_->display_id();
@@ -85,10 +84,9 @@ void DrmDisplayHost::OnHDCPStateReceived(bool status,
   get_hdcp_callback_.Reset();
 }
 
-void DrmDisplayHost::SetHDCPState(
-    display::HDCPState state,
-    const display::SetHDCPStateCallback& callback) {
-  set_hdcp_callback_ = callback;
+void DrmDisplayHost::SetHDCPState(display::HDCPState state,
+                                  display::SetHDCPStateCallback callback) {
+  set_hdcp_callback_ = std::move(callback);
   if (!sender_->GpuSetHDCPState(snapshot_->display_id(), state))
     OnHDCPStateUpdated(false);
 }
@@ -96,7 +94,7 @@ void DrmDisplayHost::SetHDCPState(
 void DrmDisplayHost::OnHDCPStateUpdated(bool status) {
   if (!set_hdcp_callback_.is_null()) {
     base::ThreadTaskRunnerHandle::Get()->PostTask(
-        FROM_HERE, base::Bind(set_hdcp_callback_, status));
+        FROM_HERE, base::BindOnce(std::move(set_hdcp_callback_), status));
   } else {
     LOG(ERROR) << "Got unexpected event for display "
                << snapshot_->display_id();
@@ -105,12 +103,15 @@ void DrmDisplayHost::OnHDCPStateUpdated(bool status) {
   set_hdcp_callback_.Reset();
 }
 
-void DrmDisplayHost::SetColorCorrection(
+void DrmDisplayHost::SetColorMatrix(const std::vector<float>& color_matrix) {
+  sender_->GpuSetColorMatrix(snapshot_->display_id(), color_matrix);
+}
+
+void DrmDisplayHost::SetGammaCorrection(
     const std::vector<display::GammaRampRGBEntry>& degamma_lut,
-    const std::vector<display::GammaRampRGBEntry>& gamma_lut,
-    const std::vector<float>& correction_matrix) {
-  sender_->GpuSetColorCorrection(snapshot_->display_id(), degamma_lut,
-                                 gamma_lut, correction_matrix);
+    const std::vector<display::GammaRampRGBEntry>& gamma_lut) {
+  sender_->GpuSetGammaCorrection(snapshot_->display_id(), degamma_lut,
+                                 gamma_lut);
 }
 
 void DrmDisplayHost::OnGpuProcessLaunched() {}
